from rest_framework import serializers
from django.contrib.auth.models import User
from oms_cms.backend.news.models import Category, Post, Tags, FilterPost


class UserSerilizer(serializers.ModelSerializer):
    '''Сериализация пользователя'''
    class Meta:
        model = User
        fields = ('id', 'username')


# Post ############################

class TagsInPostSerializer(serializers.ModelSerializer):
    '''Сериализация тегов (для отображения в новости)'''
    class Meta:
        model = Tags
        fields = ('id', 'name')


class CategoryInPostSerializer(serializers.ModelSerializer):
    '''Сериализация категории (для отображения в новости)'''
    class Meta:
        model = Category
        fields = ('id', 'name')


class PostSerializerRetrieve(serializers.ModelSerializer):
    '''Сериализация отдельной новости'''
    author = UserSerilizer(read_only=True)
    tag = TagsInPostSerializer(many=True, read_only=True)
    category = CategoryInPostSerializer()
    user_like = UserSerilizer(many=True, read_only=True)
    class Meta:
        model = Post
        fields = '__all__'


class PostListSerializer(serializers.ModelSerializer):
    '''Сериализация всех новостей'''
    author = UserSerilizer()
    class Meta:
        model = Post
        fields = '__all__'

class PostDeleteUpdateCreateSerializer(serializers.ModelSerializer):
    '''Сериализация для удаления/изменения/создания новостей'''
    class Meta:
        model = Post
        fields = '__all__'


# Category ############################

class CategoryPostsSerializer(serializers.HyperlinkedModelSerializer):
    '''Сериализация новостей (для отображения в категории)'''
    class Meta:
        model = Post
        fields = ('id', 'title', 'published')


class CategorySerializerRetrieve(serializers.ModelSerializer):
    '''Сериализация отдельной категории'''
    category_posts = CategoryPostsSerializer(many=True, read_only=True)
    class Meta:
        model = Category
        fields = ('id', 'lang', 'slug', 'name', 'title', 'description',
                  'template', 'published', 'paginated', 'sort',
                  'lft', 'rght', 'tree_id', 'level', 'parent', 'category_posts')

class CategorySerializer(serializers.ModelSerializer):
    """Сериализация категорий"""
    class Meta:
        model = Category
        fields = '__all__'
        read_only_fields = ('id',)


# Tags ############################

class TagsSerializer(serializers.ModelSerializer):
    '''Сериализация тегов'''
    class Meta:
        model = Tags
        fields = ('__all__')
        read_only_fields = ('id', )



# Generated by Django 2.2.5 on 2020-02-02 14:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('news', '0008_category_title'),
    ]

    operations = [
        migrations.AddField(
            model_name='filterpost',
            name='icon_inactive',
            field=models.FileField(blank=True, null=True, upload_to='filters/', verbose_name='Изображение не активной иконки'),
        ),
    ]



from django.urls import path
from . import views

urlpatterns = [
    path('tags-list/', views.TagsListApi.as_view()),                                        # Список тегов
    path('tags/<int:id>/', views.TagRetrieveWithId.as_view()),                              # Просмотр информации о теге (ID)
    path('tags/<slug:slug>/', views.TagRetrieveWithSlug.as_view()),                         # Просмотр информации о теге (slug)
    path('tags-delete-update/<int:id>/', views.TagsDeleteUpdateWithId.as_view()),           # Удалить/изменить тег (ID)
    path('tag-create/', views.TagsCreate.as_view()),                                        # Создание тега
    path('category-list/', views.CategoryListApi.as_view()),                                # Список категорий
    path('category/<int:id>/', views.CategoryRetrieveWithId.as_view()),                     # Просмотр информации о категории (ID)
    path('category/<slug:slug>/', views.CategoryRetrieveWithSlug.as_view()),                # Просмотр информации о категории (slug)
    path('category-delete-update/<int:id>/', views.CategoryDeleteUpdateWithId.as_view()), #FIX slug/url input   # Удалить/изменить категорию (ID)
    path('category-create/', views.CategoryCreate.as_view()),            #FIX slug/url input                   # Создание категории
    path('post-list/', views.PostList.as_view()),                                           # Список статей
    path('post/<int:id>/', views.PostRetrieveWithId.as_view()),                             # Просмотр информации о новости (ID)
    path('post/<slug:slug>/', views.PostRetrieveWithSlug.as_view()),                        # Просмотр информации о новости (slug)
    path('post-delete-update/<int:id>/', views.PostDeleteUpdateWithId.as_view()),  #FIX slug/url input         # Удалить/изменить новость (ID)
    path('post-delete-update/<slug:slug>/', views.PostDeleteUpdateWithSlug.as_view()), #FIX slug/url input     # Удалить/изменить новость (slug)
    path('post-create/', views.PostCreate.as_view()),                  #FIX slug/url input                     # Создание новости
]



from django.urls import path,  include
# from rest_framework import permissions
# from drf_yasg.views import get_schema_view
# from drf_yasg import openapi

# schema_view = get_schema_view(
#     openapi.Info(
#         title="Oms API",
#         default_version='v1',
#         description="Test description",
#         # terms_of_service="https://www.google.com/policies/terms/",
#         # contact=openapi.Contact(email="contact@snippets.local"),
#         # license=openapi.License(name="BSD License"),
#     ),
#     public=True,
#     permission_classes=(permissions.AllowAny,),
# )

urlpatterns = [
    path('news/', include('oms_cms.backend.api.v2.news.urls')),


    # re_path('swagger(?P<format>\.json|\.yaml)', schema_view.without_ui(cache_timeout=0),
    #         name='schema-json'),
    # path('swagger/', schema_view.with_ui('swagger', cache_timeout=0),
    #      name='schema-swagger-ui'),
    # path('redoc/', schema_view.with_ui('redoc', cache_timeout=0), name='schema-redoc'),
]


# Generated by Django 3.0.3 on 2020-02-29 09:41

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('news', '0009_filterpost_icon_inactive'),
    ]

    operations = [
        migrations.AlterField(
            model_name='post',
            name='category',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='category_posts', to='news.Category', verbose_name='Категория'),
        ),
    ]



from rest_framework import generics, permissions
from rest_framework import filters as filters_rf
from django_filters import rest_framework as filters

from oms_cms.backend.news.models import Category, Post, Tags
from .serializers import TagsSerializer, CategorySerializer, PostSerializerRetrieve, PostListSerializer, \
    CategorySerializerRetrieve, PostDeleteUpdateCreateSerializer


class TagsListApi(generics.ListAPIView):
    '''Список всех тегов'''
    permission_classes = [permissions.AllowAny]
    queryset = Tags.objects.all()
    serializer_class = TagsSerializer
    filter_backends = [filters.DjangoFilterBackend,
                       filters_rf.SearchFilter,
                       filters_rf.OrderingFilter]
    filter_fields = ('id', 'name', 'slug', 'published')
    search_fields = ['id', 'name', 'slug']
    ordering = ['id']


class TagRetrieveWithId(generics.RetrieveAPIView):
    '''Просмотр информации об отдельном теге (доступ через ID)'''
    permission_classes = [permissions.AllowAny]
    queryset = Tags.objects.all()
    serializer_class = TagsSerializer
    lookup_field = 'id'


class TagRetrieveWithSlug(generics.RetrieveAPIView):
    '''Просмотр информации об отдельном теге (доступ через slug)'''
    permission_classes = [permissions.AllowAny]
    queryset = Tags.objects.all()
    serializer_class = TagsSerializer
    lookup_field = 'slug'


class TagsDeleteUpdateWithId(generics.RetrieveAPIView,
                             generics.UpdateAPIView,
                             generics.DestroyAPIView):
    '''Удаление и изменение тега (доступ через ID)'''
    permission_classes = [permissions.DjangoModelPermissions]
    queryset = Tags.objects.all()
    lookup_field = 'id'
    serializer_class = TagsSerializer


class TagsCreate(generics.CreateAPIView,
                 generics.GenericAPIView):
    '''Создание тега'''
    permission_classes = [permissions.DjangoModelPermissions]
    serializer_class = TagsSerializer
    queryset = Tags.objects.none()  # Required for DjangoModelPermissions

    def post(self, request, *args, **kwargs):
        return self.create(request, *args, **kwargs)


class CategoryListApi(generics.ListAPIView):
    '''Список всех категорий'''
    permission_classes = [permissions.AllowAny]
    queryset = Category.objects.all()
    serializer_class = CategorySerializer
    filter_backends = [filters.DjangoFilterBackend,
                       filters_rf.SearchFilter,
                       filters_rf.OrderingFilter]
    filter_fields = ('id', 'lang', 'slug', 'name', 'title', 'description',
                     'template', 'published', 'paginated', 'sort',
                     'lft', 'rght', 'tree_id', 'level', 'parent')
    search_fields = ['id', 'name', 'title', 'slug']
    ordering = ['id']


class CategoryRetrieveWithId(generics.RetrieveAPIView):
    '''Просмотр информации об отдельной категории (доступ через ID)'''
    permission_classes = [permissions.AllowAny]
    queryset = Category.objects.all()
    lookup_field = 'id'
    serializer_class = CategorySerializerRetrieve



class CategoryRetrieveWithSlug(generics.RetrieveAPIView):
    '''Просмотр информации об отдельной категории (доступ через slug)'''
    permission_classes = [permissions.AllowAny]
    queryset = Category.objects.all()
    serializer_class = CategorySerializerRetrieve
    lookup_field = 'slug'


class CategoryDeleteUpdateWithId(generics.RetrieveAPIView,
                                 generics.UpdateAPIView,
                                 generics.DestroyAPIView):
    '''Удаление и изменение категории (доступ через ID)'''
    permission_classes = [permissions.DjangoModelPermissions]
    queryset = Category.objects.all()
    lookup_field = 'id'
    serializer_class = CategorySerializer


class CategoryCreate(generics.CreateAPIView,
                     generics.GenericAPIView):
    '''Создание категории'''
    permission_classes = [permissions.DjangoModelPermissions]
    serializer_class = CategorySerializer
    queryset = Category.objects.none()  # Required for DjangoModelPermissions


class PostList(generics.ListAPIView):
    '''Список всех новостей'''
    permission_classes = [permissions.AllowAny]
    queryset = Post.objects.all()
    serializer_class = PostListSerializer
    filter_backends = [filters.DjangoFilterBackend,
                       filters_rf.SearchFilter,
                       filters_rf.OrderingFilter]
    filter_fields = ('id', 'lang', 'slug', 'title', 'subtitle', 'mini_text',
                     'text', 'created_date', 'edit_date', 'published_date',
                     'template', 'published', 'viewed', 'status', 'sort',
                     'like', 'author', 'image', 'category', 'tag', 'filters',
                     'user_like')
    search_fields = ['id', 'title', 'slug']
    ordering = ['id']


class PostRetrieveWithId(generics.RetrieveAPIView):
    '''Просмотр информации об отдельной новости (доступ через ID)'''
    permission_classes = [permissions.AllowAny]
    queryset = Post.objects.all()
    serializer_class = PostSerializerRetrieve
    lookup_field = 'id'


class PostRetrieveWithSlug(generics.RetrieveAPIView):
    '''Просмотр информации об отдельной новости (доступ через slug)'''
    permission_classes = [permissions.AllowAny]
    queryset = Post.objects.all()
    serializer_class = PostSerializerRetrieve
    lookup_field = 'slug'


class PostDeleteUpdateWithId(generics.RetrieveAPIView,
                             generics.UpdateAPIView,
                             generics.DestroyAPIView):
    '''Удаление и изменение новости (доступ через ID)'''
    permission_classes = [permissions.DjangoModelPermissions]
    queryset = Post.objects.all()
    lookup_field = 'id'
    serializer_class = PostDeleteUpdateCreateSerializer


class PostDeleteUpdateWithSlug(generics.RetrieveAPIView,
                               generics.UpdateAPIView,
                               generics.DestroyAPIView):
    '''Удаление и изменение новости (доступ через slug)'''
    permission_classes = [permissions.DjangoModelPermissions]
    queryset = Post.objects.all()
    lookup_field = 'slug'
    serializer_class = PostDeleteUpdateCreateSerializer


class PostCreate(generics.CreateAPIView,
                 generics.GenericAPIView):
    """Создание новости"""
    permission_classes = [permissions.DjangoModelPermissions]
    serializer_class = PostDeleteUpdateCreateSerializer
    queryset = Post.objects.none()  # Required for DjangoModelPermissions



# Generated by Django 3.0.3 on 2020-03-03 16:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pages', '0005_auto_20191211_1756'),
    ]

    operations = [
        migrations.AddField(
            model_name='blockpage',
            name='background_color',
            field=models.CharField(blank=True, max_length=7, null=True, verbose_name='Цвет фона'),
        ),
    ]
